package Business::Price::Collection;
our $VERSION = '1.000_1';

use Moose;

__PACKAGE__->meta->make_immutable;
__END__
=pod

=head1 NAME

Business::Price::Collection

=head1 VERSION

version 1.000_1

=head1 AUTHOR

  Moritz Onken <onken@netcubed.de>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2010 by Moritz Onken.

This is free software, licensed under:

  The (three-clause) BSD License

=cut

